#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <SDL.h>
#include <SDL/SDL_ttf.h>
#include <SDL/SDL_image.h>
#include "easywiz.h"

/* The screen surface, joystick device */
SDL_Surface *screen = NULL;
SDL_Joystick *joy = NULL;
SDL_Surface *tmp_screen;
unsigned char *keys;

wz_image image[MAX_IMAGES];

wzKeyboard pad;
wzMouse mouse;

void Terminate(void)
{
	SDL_Quit();
    TTF_Quit();
#ifdef GP2X
	chdir("/usr/gp2x");
	execl("/usr/gp2x/gp2xmenu", "/usr/gp2x/gp2xmenu", NULL);
#endif
}

void wzInit() {
	if (SDL_Init (SDL_INIT_VIDEO | SDL_INIT_AUDIO | SDL_INIT_JOYSTICK | SDL_INIT_TIMER) < 0) {
		fprintf (stderr, "Couldn't initialize SDL: %s\n", SDL_GetError ());
		exit (1);
	}
	atexit (Terminate);

	SDL_ShowCursor(SDL_DISABLE);

	/* Set 320x240 16-bits video mode */
	screen = SDL_SetVideoMode(320,240,16,SDL_SWSURFACE|SDL_DOUBLEBUF);
	if (screen == NULL) {
		fprintf (stderr, "Couldn't set 320x240x16 video mode: %s\n", SDL_GetError ());
		exit (2);
	}
	
	tmp_screen = SDL_CreateRGBSurface(SDL_SWSURFACE, 320, 240, 16, 0xf800, 0x07e0, 0x001f, 0x0000);

	/* Check and open joystick device */
	joy = SDL_JoystickOpen(0);
	if(!joy) {
		fprintf (stderr, "Couldn't open joystick 0: %s\n", SDL_GetError ());
	}
	
    TTF_Init();
}

int wzUpdateInput() {
	SDL_Event event;
	int input_v = 0;
	keys = SDL_GetKeyState (NULL);
  
  int wz_done = 0;
  
#ifdef WIN32
	pad.newpress.up =  ((keys[VK_UP]) & (!pad.held.up));
	pad.newpress.down =  ((keys[VK_DOWN]) & (!pad.held.down));
	pad.newpress.left =  ((keys[VK_LEFT]) & (!pad.held.left));
	pad.newpress.right =  ((keys[VK_RIGHT]) & (!pad.held.right));
	pad.newpress.a =  ((keys[VK_FA]) & (!pad.held.a));
    pad.newpress.b =  ((keys[VK_FB]) & (!pad.held.b));
    pad.newpress.x =  ((keys[VK_FX]) & (!pad.held.x));
    pad.newpress.y =  ((keys[VK_FY]) & (!pad.held.y));
    pad.newpress.start =  ((keys[SDLK_RETURN]) & (!pad.held.start));
    pad.newpress.select =  ((keys[SDLK_SPACE]) & (!pad.held.select));
    pad.newpress.volup =  ((keys[SDLK_PAGEUP]) & (!pad.held.volup));
	pad.newpress.voldown =  ((keys[SDLK_PAGEDOWN]) & (!pad.held.voldown));
      
    pad.held.up = keys[SDLK_UP];
    pad.held.left = keys[SDLK_LEFT];
    pad.held.right = keys[SDLK_RIGHT];
    pad.held.down = keys[SDLK_DOWN];
    pad.held.a =  keys[VK_FA];
    pad.held.b =  keys[VK_FB];
    pad.held.x =  keys[VK_FX];
    pad.held.y =  keys[VK_FY];    
    pad.newpress.start =  keys[SDLK_RETURN];
    pad.newpress.select =  keys[SDLK_SPACE];
    pad.newpress.volup =  keys[SDLK_PAGEUP];
	pad.newpress.voldown =  keys[SDLK_PAGEDOWN];
      
#endif    

#ifdef GP2X    
    pad.newpress.up =  ((SDL_JoystickGetButton(joy,VK_UP)) & (!pad.held.up));
	pad.newpress.up_left =  ((SDL_JoystickGetButton(joy,VK_UP_LEFT)) & (!pad.held.up_left));
	pad.newpress.up_right =  ((SDL_JoystickGetButton(joy,VK_UP_RIGHT)) & (!pad.held.up_right));
    pad.newpress.down =  ((SDL_JoystickGetButton(joy,VK_DOWN)) & (!pad.held.down));
	pad.newpress.down_left =  ((SDL_JoystickGetButton(joy,VK_DOWN_LEFT)) & (!pad.held.down_left));
	pad.newpress.down_right =  ((SDL_JoystickGetButton(joy,VK_DOWN_RIGHT)) & (!pad.held.down_right));
    pad.newpress.left =  ((SDL_JoystickGetButton(joy,VK_LEFT)) & (!pad.held.left));
    pad.newpress.right =  ((SDL_JoystickGetButton(joy,VK_RIGHT)) & (!pad.held.right));
    pad.newpress.a =  ((SDL_JoystickGetButton(joy,VK_FA)) & (!pad.held.a));
    pad.newpress.b =  ((SDL_JoystickGetButton(joy,VK_FB)) & (!pad.held.b));
    pad.newpress.x =  ((SDL_JoystickGetButton(joy,VK_FX)) & (!pad.held.x));
    pad.newpress.y =  ((SDL_JoystickGetButton(joy,VK_FY)) & (!pad.held.y));
    pad.newpress.volup =  ((SDL_JoystickGetButton(joy,VK_VOL_UP)) & (!pad.held.volup));
	pad.newpress.voldown =  ((SDL_JoystickGetButton(joy,VK_VOL_DOWN)) & (!pad.held.voldown));
    pad.newpress.start =  ((SDL_JoystickGetButton(joy,VK_START)) & (!pad.held.start));
    pad.newpress.select =  ((SDL_JoystickGetButton(joy,VK_SELECT)) & (!pad.held.select));
     
    pad.held.up = SDL_JoystickGetButton(joy,VK_UP);
	pad.held.up_left = SDL_JoystickGetButton(joy,VK_UP_LEFT);
	pad.held.up_right = SDL_JoystickGetButton(joy,VK_UP_RIGHT);
    pad.held.left = SDL_JoystickGetButton(joy,VK_LEFT);
    pad.held.right = SDL_JoystickGetButton(joy,VK_RIGHT);
    pad.held.down = SDL_JoystickGetButton(joy,VK_DOWN);
	pad.held.down_left = SDL_JoystickGetButton(joy,VK_DOWN_LEFT);
	pad.held.down_right = SDL_JoystickGetButton(joy,VK_DOWN_RIGHT);
    pad.held.a =  SDL_JoystickGetButton(joy,VK_FA);
    pad.held.b =  SDL_JoystickGetButton(joy,VK_FB);
    pad.held.x =  SDL_JoystickGetButton(joy,VK_FX);
    pad.held.y =  SDL_JoystickGetButton(joy,VK_FY);
    pad.held.volup =  SDL_JoystickGetButton(joy,VK_VOL_UP);
    pad.held.voldown =  SDL_JoystickGetButton(joy,VK_VOL_DOWN);
    pad.held.start =  SDL_JoystickGetButton(joy,VK_START);
    pad.held.select =  SDL_JoystickGetButton(joy,VK_SELECT);
      
#endif           
    mouse.released = 0;
	mouse.newpress = 0;
	while (SDL_PollEvent (&event))
	{
			switch (event.type)
			{
				case SDL_KEYDOWN:
					/* if press Ctrl + C, terminate program */
					if ( (event.key.keysym.sym == SDLK_c) && (event.key.keysym.mod & (KMOD_LCTRL | KMOD_RCTRL)) )
						wz_done = 1;
					input_v = event.key.keysym.sym;
					
					
					break;
				case SDL_KEYUP:
					break;
				case SDL_JOYBUTTONDOWN:
					/* if press Start button, terminate program */
					if (event.key.keysym.sym == VK_START )
						wz_done = 1;
					break;
				case SDL_JOYBUTTONUP:
					break;
					
				case SDL_MOUSEMOTION:
                    mouse.x = event.button.x;
                    mouse.y = event.button.y;
                    mouse.held = 1;
                    mouse.newpress = 0;
                break;
        
                case SDL_MOUSEBUTTONDOWN:
                    switch(event.button.button) {
                        case SDL_BUTTON_LEFT:
                            mouse.newpress = 1&(!mouse.held);
                            mouse.held = 1;
                            mouse.x = event.button.x;
                            mouse.y = event.button.y;
                        break;
                    }
                break;
        
                case SDL_MOUSEBUTTONUP:
                    switch(event.button.button) {
                        case SDL_BUTTON_LEFT:
                            mouse.x = event.button.x;
                            mouse.y = event.button.y;
                            mouse.released = 1;
                            mouse.held = 0;
                        break;
                    }
                break;
				case SDL_QUIT:
					wz_done = 1;
					break;
				default:
					break;
			}
		
    }
    
    return wz_done;
}

void wzUpdateScreen() {
	SDL_BlitSurface(tmp_screen,0,screen,0);
	SDL_Flip(screen);
}

int wzLoadImage(int n, char *name) {
if(n>MAX_IMAGES) return 0;
	SDL_Surface *tmp;
	tmp = IMG_Load(name);
	image[n].img = SDL_DisplayFormatAlpha(tmp);
	SDL_FreeSurface(tmp);
	return 1;
}

void wzDisplayImage(int n, int x, int y) {
    if(!image[n].img) return;
    SDL_Rect dst;
    dst.w=image[n].img->w;
    dst.h=image[n].img->h;
    dst.x=x;
    dst.y=y;
    SDL_BlitSurface(image[n].img,0,tmp_screen,&dst);
}

void wzDisplayImagePart(int n, int x, int y, int w, int h, int posx, int posy) {
    if(!image[n].img) return;
    SDL_Rect dst;
    dst.w=w;
    dst.h=h;
    dst.x=x;
    dst.y=y;
    SDL_Rect src;
    src.w=w;
    src.h=h;
    src.x=posx;
    src.y=posy;
    SDL_BlitSurface(image[n].img,&src,tmp_screen,&dst);
}

int Collision(int ax,int ay,int aw,int ah,int bx,int by,int bw,int bh) {
    if( ax      < bx + bw  &&
		ax + aw > bx        &&
        ay      < by + bh  &&
        ay + ah > by)
    {
    return 1;
    }
    return 0;
}
